@@ -41,6 +41,10 @@ extract.psi<-function(lista){
       n<-length(y)
       o0<-try(suppressWarnings(seg.lm.fit(y, XREG, Z, PSI, w, offs, opz, return.all.sol=FALSE)), silent=TRUE)
       rangeZ <- apply(Z, 2, range) #serve sempre
+      
+      alpha <- opz$alpha
+      limZ <- apply(Z, 2, quantile, names = FALSE, probs = c(alpha, 1 - alpha))
+      
       if(!is.list(o0)) {
           o0<- suppressWarnings(seg.lm.fit(y, XREG, Z, PSI, w, offs, opz, return.all.sol=TRUE))
           o0<-extract.psi(o0)
@@ -54,7 +58,7 @@ extract.psi<-function(lista){
         } else {
           if(!nonParam) stop("the first fit failed and I cannot extract fitted values for the semipar boot")
           if(random) {
-            est.psi00<-est.psi0<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
+            est.psi00<-est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
             PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
             o0<-try(suppressWarnings(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
             ss00<-o0$SumSquares.no.gap
@@ -84,7 +88,7 @@ extract.psi<-function(lista){
         #if(length(na.omit(diff(all.selected.ss[1:n.boot.rev])))==(n.boot.rev-1) && all(round(diff(all.selected.ss[1:n.boot.rev]),6)==0)){
         if(length(diff.selected.ss)>=(n.boot.rev-1) && all(round(diff.selected.ss[1:(n.boot.rev-1)],6)==0)){
           qpsi<-sapply(1:ncol(Z),function(i)mean(est.psi0[i]>=Z[,i]))
-          qpsi<-ifelse(abs(qpsi-.5)<.1,alpha,qpsi)
+          qpsi<-ifelse(abs(qpsi-.5)<.1, alpha, qpsi)
           alpha<-1-alpha
           est.psi0<-sapply(1:ncol(Z),function(i)quantile(Z[,i],probs=1-qpsi[i],names=FALSE))
         }
@@ -103,7 +107,7 @@ extract.psi<-function(lista){
         if(is.list(o.boot)){
             all.est.psi.boot[k,]<-est.psi.boot<-o.boot$psi
         } else {
-            est.psi.boot<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
+            est.psi.boot<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
         }
         #if(k==7) browser()
         ### se est.psi.boot non e' cambiato (e puoi vederlo da all.est.psi.boot), allora cambialo!
@@ -114,7 +118,7 @@ extract.psi<-function(lista){
         opz$it.max<-opz$it.max+1
         o<-try(suppressWarnings(seg.lm.fit(y, XREG, Z.orig, PSI, w, offs, opz, return.all.sol=TRUE)), silent=TRUE)
         if(!is.list(o) && random){
-                est.psi0<-apply(rangeZ,2,function(r)runif(1,r[1],r[2]))
+                est.psi0<-apply(limZ,2,function(r)runif(1,r[1],r[2]))
                 PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
                 o<-try(suppressWarnings(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
                 count.random<-count.random+1
@@ -168,7 +172,7 @@ extract.psi<-function(lista){
 
       if(is.null(o0$obj)){
           PSI1 <- matrix(rep(est.psi0, rep(nrow(Z), length(est.psi0))), ncol = length(est.psi0))
-          o0<-try(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1), silent=TRUE)
+          o0<-try(suppressWarnings(seg.lm.fit(y, XREG, Z, PSI1, w, offs, opz1)), silent=TRUE)
       }
       if(!is.list(o0)) return(0)
       o0$boot.restart<-ris
